import argparse
import sys
import os
from glob import glob
from .core import VideoProcessor
from .utils import (
    get_ffmpeg_path,
    print_error,
    print_warning,
    print_success
)

def main():
    try:
        parser = argparse.ArgumentParser(
            usage="revidx INPUTFILES ... [OPTIONS]",
            description="reVidx: Convert HEVC to AVC and Audio to MP3/AAC.",
            epilog="Requires 'ffmpeg' and 'ffprobe'.\n"
            "Docs & issues: https://github.com/voidsnax/reVidx",
            formatter_class=argparse.RawTextHelpFormatter
        )

        parser.add_argument('inputfiles', nargs='+', help='Input video files')
        parser.add_argument('-o',metavar='PATH/NAME' ,nargs='?', const=None, help='Output directory or filename')
        parser.add_argument('-skip', action='store_true', help='Skip video encoding (copy)')
        parser.add_argument('-burn', metavar='INDEX/PATH',nargs='?', const='DEFAULT', help='Burn subtitles into the video (default: 0)')
        parser.add_argument('-aindex', metavar='INDEX',type=int, help='Audio index (default: 0)')
        parser.add_argument('-audio', action='store_true', help='Extract only audio to AAC')
        parser.add_argument('-crf', metavar='VALUE',type=float, help='CRF value (default: 20)')

        args = parser.parse_args()

        # Check Dependencies
        ffmpeg_path, ffprobe_path = get_ffmpeg_path()
        if not ffmpeg_path or not ffprobe_path:
            print_error("'ffmpeg' and 'ffprobe' not found.")
            sys.exit(1)

        processor = VideoProcessor(ffmpeg_path, ffprobe_path)

        multi_input = len(args.inputfiles) > 1
        if multi_input and args.burn != 'DEFAULT':
            print_error("Cannot provide subtitle stream for multiple inputs.")
            print_warning("Leave it empty for using the default.")
            sys.exit(1)

        # Options
        options = {
            'skip_video': args.skip,
            'burn': args.burn,
            'aindex': args.aindex if args.aindex is not None else 0,
            'audio_only': args.audio,
            'crf': args.crf if args.crf is not None else 20
        }

        if multi_input and args.burn:
            print_warning("Processing multiple files with -burn. Default subtitles used.")

        input_files = []
        counter = 0

        for input_arg in args.inputfiles:
            input_files.extend(glob(input_arg))
        if not input_files:
            print_error(f"No valid files found")
            sys.exit(1)

        total_files = len(input_files)

        # Process
        for file in input_files:
            if os.path.isdir(file):
                print_error(f"{file} is a directory")
                sys.exit(1)

            counter += 1
            base_name = os.path.splitext(os.path.basename(file))[0]

            output_video_path = ""
            output_audio_path = ""

            if args.o:
                if multi_input:
                    if not os.path.isdir(args.o):
                        print_error(f"With multiple inputs, -o must be a directory.")
                        sys.exit(1)
                    out_dir = args.o
                    output_video_path = os.path.join(out_dir, f"{base_name}-AvcMp3.mp4")
                    output_audio_path = os.path.join(out_dir, f"{base_name}.aac")
                else:
                    if os.path.isdir(args.o):
                        output_video_path = os.path.join(args.o, f"{base_name}-AvcMp3.mp4")
                        output_audio_path = os.path.join(args.o, f"{base_name}.aac")
                    else:
                        if args.audio and not args.burn:
                            output_audio_path = args.o
                            output_video_path = None
                        else:
                            output_video_path = args.o
                            if args.audio:
                                output_audio_path = os.path.splitext(args.o)[0] + ".aac"
            else:
                output_video_path = f"{base_name}-AvcMp3.mp4"
                output_audio_path = f"{base_name}.aac"

            output_config = {
                'video': output_video_path,
                'audio': output_audio_path
            }

            cmd = processor.construct_commands(file, output_config, options)

            out_file = cmd[-1]
            # print(*cmd)
            success = processor.run_ffmpeg(cmd, file, os.path.basename(file), total_files, counter)

            if success:
                print_success(f">> ",end='')
                print(f"{out_file}")
            else:
                print_error(f">> ",end='')
                print(f"{out_file}")

            if not counter == total_files:
                print()

    except KeyboardInterrupt:
        print_warning("\n>> ",end='')
        print(f"{out_file}")
        print_error("\nProcess Aborted")
        sys.exit(1)

if __name__ == "__main__":
    main()


import subprocess
import sys
import time
import os
from colorama import Fore, Style
from .utils import (
    escape_path_for_filter,
    print_error,
    print_warning,
    print_info,
    format_bytes,
    format_seconds_hms,
    is_android
)


class VideoProcessor:
    def __init__(self, ffmpeg_path="ffmpeg", ffprobe_path="ffprobe"):
        self.ffmpeg = ffmpeg_path
        self.ffprobe = ffprobe_path

    def get_total_duration(self, input_path):
        """Gets total duration of the input video."""
        cmd = [
            self.ffprobe, "-v", "error",
            "-show_entries", "format=duration",
            "-of", "default=noprint_wrappers=1:nokey=1",
            input_path
        ]
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, check=True)
            return float(result.stdout.strip())
        except (subprocess.CalledProcessError, ValueError):
            return 0.0

    def run_ffmpeg(self, args, input_path, filename, file_count, counter):
        """
        Runs FFmpeg.
        """
        cmd = [self.ffmpeg] + args
        cmd.extend(['-progress', 'pipe:1', '-nostats', '-v', 'error'])

        try:
            process = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                encoding='utf-8',
                errors='ignore'
            )

            total_duration = self.get_total_duration(input_path)

            # --- Spinner Setup ---
            spinners = ['|', '/', '-', '\\']
            spinner_idx = 0

            print_info(f"<< ",end='')
            print(f"{filename}")

            if file_count > 1:
                print(f"{Fore.GREEN}{counter}{Style.RESET_ALL} of {Fore.BLUE}{file_count}{Style.RESET_ALL}")
            else:
                print()

            # If duration is 0
            if total_duration == 0.0:
                print_error("Duration unknown.File Error")
                return False

            else:
                current_time = 0.0
                total_size = 0
                start_clock = time.time()
                last_display_time = 0

                while True:
                    if process.stdout:
                        line = process.stdout.readline()

                    if not line and process.poll() is not None:
                        break

                    if '=' in line:
                        key, value = line.strip().split('=', 1)

                        if key == 'out_time_us':
                            current_time = int(value) / 1_000_000

                        elif key == 'total_size':
                            total_size = int(value)

                        elif key == 'progress' and value == 'end':
                            percent = 100
                            elapsed_wall_clock = time.time() - start_clock
                            size_str = format_bytes(total_size)
                            cur_str = format_seconds_hms(current_time)

                            if is_android():
                                status = (
                                    f"{cur_str} {Fore.YELLOW}{size_str:>9}{Style.RESET_ALL}"
                                    f"{Fore.GREEN}{percent:>8}%{Style.RESET_ALL}   "
                                    f"✓ {elapsed_str}"
                                )
                            else:
                                status = (
                                    f"[ {Fore.GREEN}✓{Style.RESET_ALL} ] "
                                    f"Size: {Fore.YELLOW}{size_str:>9}{Style.RESET_ALL}    {"Time: "}{cur_str}/{tot_str}"
                                    f"{Fore.GREEN}{percent:>8}%{Style.RESET_ALL}    "
                                    f"{"Elapsed: "}{elapsed_str}"
                                )

                            print(f"\r{status}", end='')
                            break

                    # Update display every 0.5s
                    now = time.time()
                    if (now - last_display_time > 0.5) and (current_time > 0):
                        last_display_time = now
                        elapsed_wall_clock = now - start_clock
                        percent = (current_time / total_duration) * 100

                        cur_str = format_seconds_hms(current_time)
                        tot_str = format_seconds_hms(total_duration)
                        elapsed_str = format_seconds_hms(elapsed_wall_clock)

                        size_str = format_bytes(total_size)
                        spinner_idx = (spinner_idx + 1) % 4
                        spinner = spinners[spinner_idx]

                        if is_android():
                            # Time | Size | % | Spinner | Elapsed
                            status = (
                                f"{cur_str} {Fore.YELLOW}{size_str:>9}{Style.RESET_ALL}"
                                f"{Fore.GREEN}{percent:>8.1f}%{Style.RESET_ALL}   "
                                f"{Fore.BLUE}{spinner}{Style.RESET_ALL} {elapsed_str}"
                            )

                        else:
                            # Spinner | Size | Time/Total | % | Elapsed
                            status = (
                                f"[ {Fore.BLUE}{spinner}{Style.RESET_ALL} ] "
                                f"Size: {Fore.YELLOW}{size_str:>9}{Style.RESET_ALL}    {"Time: "}{cur_str}/{tot_str}"
                                f"{Fore.GREEN}{percent:>8.1f}%{Style.RESET_ALL}    "
                                f"{"Elapsed: "}{elapsed_str}"
                            )

                        print(f"\r{status}", end='')
                        sys.stdout.flush()

                print()

            # --- Check Result ---
            if process.returncode is None:
                process.wait()

            if process.returncode == 0:
                return True
            else:
                if process.stderr:
                    stderr_output = process.stderr.read()
                    print_error(f"FFmpeg exited with error code")
                    print_warning(f"{stderr_output}")
                return False

        except KeyboardInterrupt:
            process.kill()
            raise

        except Exception as e:
            print_error(f"Error occured while processing file")
            print_error(f"{e}")
            if process.poll() is None:
                process.kill()
            return False

    def construct_commands(self, input_path, output_config, options):
        """
        Constructs the ffmpeg argument list.Returns a list.
        """
        a_idx = options.get('aindex')
        audio_map = ['-map', f'0:a:{a_idx}']

        # --- Burn Logic ---
        vf_filter = ""
        if options.get('burn'):
            burn_val = options['burn']
            if burn_val == 'DEFAULT':
                sub_path = escape_path_for_filter(input_path)
                vf_filter = f"subtitles={sub_path}"
            elif os.path.exists(burn_val):
                sub_path = escape_path_for_filter(burn_val)
                vf_filter = f"subtitles={sub_path}"
            else:
                sub_path = escape_path_for_filter(input_path)
                vf_filter = f"subtitles={sub_path}:si={burn_val}"

        if options.get('audio_only'):
            output_file = output_config['audio']
            cmd = ['-y', '-i', input_path]
            cmd.extend([
                '-vn',
                '-c:a', 'aac',
                '-b:a', '128k',
                *audio_map
            ])
            cmd.append(output_file)

        else:
            output_file = output_config['video']
            cmd = ['-y', '-i', input_path, '-map', '0:v:0']

            if options.get('skip_video'):
                cmd.extend(['-c:v', 'copy'])
            else:
                crf_val = options.get('crf')
                cmd.extend([
                    '-c:v', 'libx264',
                    '-crf', str(crf_val),
                    '-preset', 'veryfast',
                    '-pix_fmt', 'yuv420p',
                    '-color_range', 'tv'
                ])
                if vf_filter: cmd.extend(['-vf', vf_filter])

            cmd.extend([
                '-c:a', 'libmp3lame',
                '-b:a', '192k',
                '-ac', '2',
                '-af', 'loudnorm=I=-16:LRA=5:TP=-1.5',
                *audio_map
            ])
            cmd.append(output_file)

        return cmd


import os
import platform
from colorama import Fore, Style, init

# Initialize colorama
init(autoreset=True)

def get_ffmpeg_path():
    """Returns ffmpeg and ffprobe paths."""
    from shutil import which
    ffmpeg = which("ffmpeg")
    ffprobe = which("ffprobe")
    if not ffmpeg or not ffprobe:
        return None,None
    return ffmpeg, ffprobe

def escape_path_for_filter(path):
    """Escapes a file path for use in ffmpeg filtergraphs."""
    abs_path = os.path.abspath(path)
    escaped = abs_path.replace("\\", "/")
    if platform.system() == "Windows":
        escaped = escaped.replace(":", "\\:")
    return f"'{escaped}'"

def format_bytes(size_bytes):
    """Converts bytes to KB/MB/GB dynamically."""
    if size_bytes == 0: return "0B"
    units = ['B', 'KB', 'MB', 'GB', 'TB']
    i = 0
    while size_bytes >= 1024 and i < len(units)-1:
        size_bytes /= 1024
        i += 1
    return f"{size_bytes:.2f}{units[i]}"

def format_seconds_hms(seconds):
    """Converts float seconds to HH:MM:SS string."""
    m, s = divmod(int(seconds), 60)
    h, m = divmod(m, 60)
    return f"{h:02d}:{m:02d}:{s:02d}"

# --- Colored Output Helpers ---
def print_error(msg,end='\n'): print(f"{Fore.RED}{msg}",end=end)

def print_success(msg,end='\n',flush=False): print(f"{Fore.GREEN}{msg}",end=end,flush=flush)

def print_warning(msg,end='\n',flush=False): print(f"{Fore.YELLOW}{msg}",end=end,flush=flush)

def print_info(msg,end='\n'): print(f"{Fore.CYAN}{msg}",end=end)

def is_android():
    import sys
    return hasattr(sys, 'getandroidapilevel')
